#!/bin/bash
# (c) wiimm (at) wiimm.de -- 2013-10-01

FAST=
[[ $WSZST_FAST = 1 || $1 == --fast ]] && FAST=--fast

#
#------------------------------------------------------------------------------
# infos

# 'ln' creates a hard link (faster than copying)
# 'ln -f' same but remove the destination first to avoid conflicts

# include.fst not needed (ignore .svn entries and all other hidden files)
rm -f include.fst

#
#------------------------------------------------------------------------------
# first give mkw-fun 100 a change to prepare

[[ -s ./patch-100.sh ]] && bash ./patch-100.sh "$@"

#
#------------------------------------------------------------------------------
# create SZS files using WU8 files

if [[ -f ./wu8.list && -d ./wu8-files ]]
then
    echo " * Create patch library"
    wszst autoadd -q ./files/Race/Course/ -D ./auto-add

    echo " * Convert $(wc -l <wu8.list) WU8 files (takes a long time)"

    count_wu8=0
    count_szs=0
    while IFS='|' read dir wu8 szs1 szs2
    do
	(( count_wu8>0 && !(count_wu8%10) )) && \
		printf "    - %u WU8 files processed, %u SZS files created\n" \
			$count_wu8 $count_szs

	wszst compress $FAST --szs "./wu8-files/$wu8" -orqD "$dir/$szs1"
	let count_wu8++
	let count_szs++
	if [[ $szs2 != "" ]]
	then
	    ln -f "$dir/$szs1" "$dir/$szs2"
	    let count_szs++
	fi
    done <./wu8.list
    printf "    - %u SZS files created\n" $count_szs
fi

#
#------------------------------------------------------------------------------
# patch BMG

bmg0="./files/_Wiimm/bmg-patch"
bmg="$bmg0/$CFG_MSGLANG"
[[ -d $bmg ]] || bmg="$bmg0/en"

if [[ -d $bmg ]]
then
    echo " * Patch BMG messages [$CFG_MSGLANG,$REV_MODE]"
    count=0
    for szs in ./files/Scene/UI/*_?.szs
    do
	obj="${szs##*/}"
	obj="${obj%.szs}"
	lang="${obj##*_}"
	obj="${obj%_?}"
	#echo "|$obj|$lang"

	pat=()
	[[ -f $bmg/mark.txt ]] && pat=( "${pat[@]}" --patch-bmg "print=$bmg/mark.txt" )

	[[ $REV_MODE = 100 ]] && pat=( "${pat[@]}" --patch-bmg ctcopy --patch-bmg ctfill )

	[[ $REV_WIIMMFI = 1 && -f $bmg0/wiimmfi/wiimmfi-$lang.txt ]] \
	    && pat=( "${pat[@]}" --patch-bmg "replace=$bmg0/wiimmfi/wiimmfi-$lang.txt" )

	if [[ -f "$bmg/$obj-$lang.txt" ]]
	then
	    pat=( "${pat[@]}" --patch-bmg "replace=$bmg/$obj-$lang.txt" )
	elif [[ -f "$bmg/$obj.txt" ]]
	then
	    pat=( "${pat[@]}" --patch-bmg "replace=$bmg/$obj.txt" )
	elif [[ -f "$bmg/all-$lang.txt" ]]
	then
	    pat=( "${pat[@]}" --patch-bmg "replace=$bmg/all-$lang.txt" )
	elif [[ -f "$bmg/all.txt" ]]
	then
	    pat=( "${pat[@]}" --patch-bmg "replace=$bmg/all.txt" )
	fi

	wszst patch $FAST -qr "$szs" "${pat[@]}"
	let count++
    done
    printf "    - %u SZS files patched\n" $count
fi

#
#------------------------------------------------------------------------------
# copy strap files

find ./files/Boot/Strap -type f -name '*.szs' | \
(
    count=0
    while read dest
    do
	lang="${dest%/*}"
	lang="${lang##*/}"
	src="./img-patch/strap-$lang.szs"
	#echo "|$lang|$src|$dest|"
	if [[ -f $src ]]
	then
	    ((count)) || echo " * Copy strap files"
	    ln -f "$src" "$dest"
	    let count++
	fi
    done
    ((count)) && printf "    - %u strap files created\n" $count
)

#
#------------------------------------------------------------------------------
# patch title images

title1="./img-patch/title1.png"
title2="./img-patch/title2.png"
titlefile=files/Scene/UI/Title.szs

if [[ -f $title1 && -f $title2 && -f $titlefile ]]
then
    echo " * Patch title images"

    work="./img-patch/title.d"
    pre="$work/title/timg/tt_title_screen_"
    rm -rf "$work"
    wszst extract -q --raw "$titlefile" -D "$work"

    count=0
    for t in mario mario0 mario2 luigi peachi koopa
    do
	f="${pre}${t}.tpl"
	if [[ -s $f ]]
	then
	    ((verbose)) && printf "   - Patch image %s\n" "$f"
	    wimgt convert -q "$f" -P "fg=$title1"
	    let count++
	fi

	f="${pre}${t}_bokeboke.tpl"
	if [[ -s $f ]]
	then
	    ((verbose)) && printf "   - Patch image %s\n" "$f"
	    wimgt convert -q "$f" -P "fg=$title2"
	    let count++
	fi
    done

    printf "    - %u title screens patched\n" $count
    ((count)) && wszst update -orq "$titlefile" -s "$work" $FAST
fi

#
#------------------------------------------------------------------------------
# replace videos to reduce image size

echo " * Replace videos to reduce image size"

v1=./files/thp/title
v2=./files/thp/ending
vsrc=$v1/top_menu.thp

if [[ -f $vsrc ]]
then
    for f in $v1/*.thp $v2/*.thp
    do
	[[ -f $f && $vsrc != $f ]] && ln -f "$vsrc" "$f"
    done
fi

#
#------------------------------------------------------------------------------
# patch main.dol && StaticR.rel

staticr="./files/rel/StaticR.rel"
maindol="./sys/main.dol"

if [[ $REV_VS != "" || $REV_BT != "" ]]
then
    vers=$(wstrt version -l | awk -F= '$1=="version" {print $2;exit}')
    need=1.37a
    if [[ $vers < $need ]]
    then
	echo "   - Patching vs/bt classes not supported, need wstrt v$need, but have only v$vers"
	REV_VS=
	REV_BT=
    fi
fi

if [[ $REV_POINTS != "" ]]
then
    vers=$(wstrt version -l | awk -F= '$1=="version" {print $2;exit}')
    need=1.36a
    if [[ $vers < $need ]]
    then
	echo "   - Patching points not supported, need wstrt v$need, but have only v$vers"
	REV_POINTS=
    fi
fi

if ((0)) && [[ -f ./$srcid/StaticR.rel ]]
then
    echo " * Copy ./$srcid/StaticR.rel"
    cp "./$srcid/StaticR.rel" "$staticr"

elif [[ $WSTRT_OPTIONS != "" || $REV_REGION != "" || $REV_POINTS != "" || $REV_HTTPS != "" ]]
then
    str_info=
    str_opt=--all-ranks
    main_info=
    main_opt=

    if [[ $WSTRT_OPTIONS != "" ]]
    then
	str_info="$str_info, track order"
    fi

    if [[ $REV_REGION != "" ]]
    then
	str_info="$str_info, region $REV_REGION"
	str_opt="$str_opt --region=$REV_REGION"
    fi

    if [[ $REV_VS != "" ]]
    then
	str_info="$str_info, vs"
	str_opt="$str_opt --vs=$REV_VS"
    fi

    if [[ $REV_BT != "" ]]
    then
	str_info="$str_info, bt"
	str_opt="$str_opt --bt=$REV_BT"
    fi

    if [[ $REV_POINTS != "" ]]
    then
	str_info="$str_info, points"
	str_opt="$str_opt --points=$REV_POINTS"
    fi

    if [[ $REV_WIIMMFI = 1 ]]
    then
	str_info="$str_info, wiimmfi"
	str_opt="$str_opt --wiimmfi"
	main_info="$main_info, wiimmfi"
	main_opt="$main_opt --wiimmfi"
    elif [[ $REV_WIIMMFI = 2 ]]
    then
	str_info="$str_info, twiimmfi"
	str_opt="$str_opt --twiimmfi"
	main_info="$main_info, twiimmfi"
	main_opt="$main_opt --twiimmfi"
    fi

    if [[ $REV_HTTPS != "" ]]
    then
	str_info="$str_info, https $REV_HTTPS"
	str_opt="$str_opt --https=$REV_HTTPS"
	main_info="$main_info, https $REV_HTTPS"
	main_opt="$main_opt --https=$REV_HTTPS"

	if [[ $REV_DOMAIN != "" ]]
	then
	    str_info="$str_info $REV_DOMAIN"
	    str_opt="$str_opt --domain=$REV_DOMAIN"
	    main_info="$main_info $REV_DOMAIN"
	    main_opt="$main_opt --domain=$REV_DOMAIN"
	fi
    fi

    echo " * Patch StaticR.rel [${str_info:2}]"
    export WIIMMS_SZS_TOOLS="$WSTRT_OPTIONS"
    wstrt patch -qr "$staticr" $WSTRT_OPTIONS $str_opt || exit 1

    if [[ $main_opt != "" ]]
    then
	echo " * Patch main.dol [${main_info:2}]"
	wstrt patch -qr "$maindol" $main_opt || exit 1

	if [[ $riivmode != "" ]] && (( riivmode > 0 ))
	then
	    mkdir -p ./files/sys
	    ln -f "$maindol" ./files/sys/
	fi
    fi
fi

#
#------------------------------------------------------------------------------
# call additonal scripts

[[ -s ./assign-mkw-sound.sh ]] && bash ./assign-mkw-sound.sh ./mkw-sound.tab files/sound/strm
[[ -s ./patch-img.sh ]] && bash ./patch-img.sh $REV_SUPPORT $CFG_MSGLANG "$@"

#
#------------------------------------------------------------------------------
# exit with success status

true

